@@ -2,8 +2,6 @@
 # Copyright: (c) 2019, Jordan Borean (@jborean93) <jborean93@gmail.com>
 # MIT License (see LICENSE or https://opensource.org/licenses/MIT)
 
-from __future__ import division
-
 import collections
 import errno
 import io
@@ -30,7 +28,6 @@ from smbprotocol import (
 
 from smbprotocol._text import (
     to_bytes,
-    to_native,
     to_text,
 )
 
@@ -121,6 +118,16 @@ SMBStatVolumeResult = collections.namedtuple('SMBStatVolumeResult', [
 ])
 
 
+def is_remote_path(path):  # type: (str) -> bool
+    """
+    Returns True iff the given path is a remote SMB path (rather than a local path).
+
+    :param path: The filepath.
+    :return: True iff the given path is a remote SMB path.
+    """
+    return path.startswith('\\\\') or path.startswith('//')
+
+
 def copyfile(src, dst, **kwargs):
     """
     Copy a file to a different location on the same server share. This will fail if the src and dst paths are to a
@@ -136,10 +143,10 @@ def copyfile(src, dst, **kwargs):
     norm_src = ntpath.normpath(src)
     norm_dst = ntpath.normpath(dst)
 
-    if not norm_src.startswith('\\\\'):
+    if not is_remote_path(norm_src):
         raise ValueError("src must be an absolute path to where the file should be copied from.")
 
-    if not norm_dst.startswith('\\\\'):
+    if not is_remote_path(norm_dst):
         raise ValueError("dst must be an absolute path to where the file should be copied to.")
 
     src_root = ntpath.splitdrive(norm_src)[0]
@@ -189,7 +196,7 @@ def copyfile(src, dst, **kwargs):
 
 def link(src, dst, follow_symlinks=True, **kwargs):
     """
-    Create a hard link pointing to src named dst. The src argument must be an absolute path in the same share as
+    Create a hard link pointing to src named dst. The dst argument must be an absolute path in the same share as
     src.
 
     :param src: The full UNC path to used as the source of the hard link.
@@ -200,7 +207,7 @@ def link(src, dst, follow_symlinks=True, **kwargs):
     norm_src = ntpath.normpath(src)
     norm_dst = ntpath.normpath(dst)
 
-    if not norm_src.startswith('\\\\'):
+    if not is_remote_path(norm_src):
         raise ValueError("src must be the absolute path to where the file is hard linked to.")
 
     src_root = ntpath.splitdrive(norm_src)[0]
@@ -407,8 +414,7 @@ def readlink(path, **kwargs):
     reparse_buffer = _get_reparse_point(norm_path, **kwargs)
     reparse_tag = reparse_buffer['reparse_tag']
     if reparse_tag.get_value() != ReparseTags.IO_REPARSE_TAG_SYMLINK:
-        raise ValueError(to_native("Cannot read link of reparse point with tag %s at '%s'" % (str(reparse_tag),
-                                                                                              norm_path)))
+        raise ValueError("Cannot read link of reparse point with tag %s at '%s'" % (str(reparse_tag), norm_path))
 
     symlink_buffer = SymbolicLinkReparseDataBuffer()
     symlink_buffer.unpack(reparse_buffer['data_buffer'].get_value())
@@ -516,13 +522,15 @@ def scandir(path, search_pattern="*", **kwargs):
     :param kwargs: Common SMB Session arguments for smbclient.
     :return: An iterator of DirEntry objects in the directory.
     """
+    connection_cache = kwargs.get('connection_cache', None)
     with SMBDirectoryIO(path, share_access='rwd', **kwargs) as fd:
         for dir_info in fd.query_directory(search_pattern, FileInformationClass.FILE_ID_FULL_DIRECTORY_INFORMATION):
             filename = dir_info['file_name'].get_value().decode('utf-16-le')
             if filename in [u'.', u'..']:
                 continue
 
-            dir_entry = SMBDirEntry(SMBRawIO(u"%s\\%s" % (path, filename), **kwargs), dir_info)
+            dir_entry = SMBDirEntry(SMBRawIO(u"%s\\%s" % (path, filename), **kwargs), dir_info,
+                                    connection_cache=connection_cache)
             yield dir_entry
 
 
@@ -648,13 +656,13 @@ def symlink(src, dst, target_is_directory=False, **kwargs):
     :param kwargs: Common SMB Session arguments for smbclient.
     """
     norm_dst = ntpath.normpath(dst)
-    if not norm_dst.startswith('\\\\'):
+    if not is_remote_path(norm_dst):
         raise ValueError("The link dst must be an absolute UNC path for where the link is to be created")
 
     norm_src = ntpath.normpath(src)
     print_name = norm_src
 
-    if not norm_src.startswith('\\\\'):
+    if not is_remote_path(norm_src):
         flags = SymbolicLinkFlags.SYMLINK_FLAG_RELATIVE
         substitute_name = norm_src
         dst_dir = ntpath.dirname(norm_dst)
@@ -666,8 +674,7 @@ def symlink(src, dst, target_is_directory=False, **kwargs):
     src_drive = ntpath.splitdrive(norm_src)[0]
     dst_drive = ntpath.splitdrive(norm_dst)[0]
     if src_drive.lower() != dst_drive.lower():
-        raise ValueError(to_native("Resolved link src root '%s' must be the same as the dst root '%s'"
-                                   % (src_drive, dst_drive)))
+        raise ValueError("Resolved link src root '%s' must be the same as the dst root '%s'" % (src_drive, dst_drive))
 
     try:
         src_stat = stat(norm_src, **kwargs)
@@ -1027,24 +1034,43 @@ def _get_reparse_point(path, **kwargs):
 
 def _rename_information(src, dst, replace_if_exists=False, **kwargs):
     verb = 'replace' if replace_if_exists else 'rename'
-    norm_src = ntpath.normpath(src)
-    norm_dst = ntpath.normpath(dst)
-
-    if not norm_dst.startswith('\\\\'):
+    if not is_remote_path(ntpath.normpath(dst)):
         raise ValueError("dst must be an absolute path to where the file or directory should be %sd." % verb)
 
-    src_root = ntpath.splitdrive(norm_src)[0]
-    dst_root, dst_name = ntpath.splitdrive(norm_dst)
-    if src_root.lower() != dst_root.lower():
-        raise ValueError("Cannot %s a file to a different root than the src." % verb)
+    raw_args = dict(kwargs)
+    raw_args.update({
+        'mode': 'r',
+        'share_access': 'rwd',
+        'create_options': CreateOptions.FILE_OPEN_REPARSE_POINT,
+    })
 
-    raw = SMBRawIO(src, mode='r', share_access='rwd', desired_access=FilePipePrinterAccessMask.DELETE,
-                   create_options=CreateOptions.FILE_OPEN_REPARSE_POINT, **kwargs)
-    with SMBFileTransaction(raw) as transaction:
-        file_rename = FileRenameInformation()
-        file_rename['replace_if_exists'] = replace_if_exists
-        file_rename['file_name'] = to_text(dst_name[1:])  # dst_name has \ prefix from splitdrive, we remove that.
-        set_info(transaction, file_rename)
+    # We open/close the dest (ignoring if the file does not exist) so we can resolve the DFS path and determine the
+    # filename src needs to be renamed to.
+    dst_raw = SMBRawIO(dst, desired_access=FilePipePrinterAccessMask.FILE_EXECUTE, **raw_args)
+    try:
+        SMBFileTransaction(dst_raw).commit()
+    except SMBOSError as err:
+        if err.errno != errno.ENOENT:
+            raise
+
+    # We need to open source first so we can get the resolved tree to compare the volumes
+    with SMBRawIO(src, desired_access=FilePipePrinterAccessMask.DELETE, **raw_args) as src_raw:
+        # We compare the server part using the GUID in case a different alias was specified for the server. The GUID
+        # should uniquely identify the server for our cases here.
+        src_guid = src_raw.fd.connection.server_guid
+        src_share = ntpath.normpath(src_raw.fd.tree_connect.share_name).split("\\")[-1]
+
+        dst_guid = dst_raw.fd.connection.server_guid
+        dst_share = ntpath.normpath(dst_raw.fd.tree_connect.share_name).split("\\")[-1]
+
+        if src_guid != dst_guid or src_share.lower() != dst_share.lower():
+            raise ValueError("Cannot %s a file to a different root than the src." % verb)
+
+        with SMBFileTransaction(src_raw) as transaction:
+            file_rename = FileRenameInformation()
+            file_rename['replace_if_exists'] = replace_if_exists
+            file_rename['file_name'] = to_text(dst_raw.fd.file_name)
+            set_info(transaction, file_rename)
 
 
 def _set_basic_information(path, creation_time=0, last_access_time=0, last_write_time=0, change_time=0,
@@ -1064,14 +1090,15 @@ def _set_basic_information(path, creation_time=0, last_access_time=0, last_write
 
 class SMBDirEntry(object):
 
-    def __init__(self, raw, dir_info):
+    def __init__(self, raw, dir_info, connection_cache=None):
         self._smb_raw = raw
         self._dir_info = dir_info
         self._stat = None
         self._lstat = None
+        self._connection_cache = connection_cache
 
     def __str__(self):
-        return '<{0}: {1!r}>'.format(self.__class__.__name__, to_native(self.name))
+        return '<{0}: {1!r}>'.format(self.__class__.__name__, self.name)
 
     @property
     def name(self):
@@ -1179,28 +1206,28 @@ class SMBDirEntry(object):
         if follow_symlinks:
             if not self._stat:
                 if self.is_symlink():
-                    self._stat = stat(self.path)
+                    self._stat = stat(self.path, connection_cache=self._connection_cache)
                 else:
                     # Because it's not a symlink lstat will be the same as stat so set both.
                     if self._lstat is None:
-                        self._lstat = lstat(self._smb_raw.name)
+                        self._lstat = lstat(self._smb_raw.name, connection_cache=self._connection_cache)
                     self._stat = self._lstat
             return self._stat
         else:
             if not self._lstat:
-                self._lstat = lstat(self.path)
+                self._lstat = lstat(self.path, connection_cache=self._connection_cache)
             return self._lstat
 
     @classmethod
-    def from_path(cls, path, **kwargs):
-        file_stat = stat(path, **kwargs)
+    def from_path(cls, path, follow_symlinks=True, **kwargs):
+        file_stat = stat(path, follow_symlinks=follow_symlinks, **kwargs)
 
         # A DirEntry only needs these 2 properties to be set
         dir_info = FileIdFullDirectoryInformation()
         dir_info['file_attributes'] = file_stat.st_file_attributes
         dir_info['file_id'] = file_stat.st_ino
 
-        dir_entry = cls(SMBRawIO(path, **kwargs), dir_info)
+        dir_entry = cls(SMBRawIO(path, **kwargs), dir_info, connection_cache=kwargs.get('connection_cache', None))
         dir_entry._stat = file_stat
         return dir_entry
 
