@@ -5,7 +5,6 @@
 import atexit
 import logging
 import ntpath
-import six
 import uuid
 import warnings
 
@@ -55,18 +54,28 @@ class _ConfigSingleton(type):
 
     def __call__(cls, *args, **kwargs):
         if cls not in cls.__instances:
-            cls.__instances[cls] = super(_ConfigSingleton, cls).__call__(*args, **kwargs)
+            config = super(_ConfigSingleton, cls).__call__(*args, **kwargs)
+            cls.__instances[cls] = config
+
+            # Needs to be done after the config instance is in the singleton dict due to setting this value will kick
+            # off an IPC connection which then gets and instance of this config to see what the default creds are.
+            # The property will not run the DFS request if the passed in value matches what is already there. Set
+            # to None initially to ensure it detects a changed value.
+            # https://github.com/jborean93/smbprotocol/issues/109
+            dc_value = config.domain_controller
+            config.domain_controller = None
+            config.domain_controller = dc_value
 
         else:
             # Allow users to initialise and set multiple config options like ConfigType(key=value) even when the object
             # has been initialized already.
-            cls.__instances[cls].set(**kwargs)
+            config = cls.__instances[cls]
+            config.set(**kwargs)
 
-        return cls.__instances[cls]
+        return config
 
 
-@six.add_metaclass(_ConfigSingleton)
-class ClientConfig(object):
+class ClientConfig(object, metaclass=_ConfigSingleton):
     """SMB Client global settings
 
     This class defines global settings for the client that affects all connections that the client makes. When setting
@@ -97,13 +106,10 @@ class ClientConfig(object):
         self.skip_dfs = skip_dfs
         self.auth_protocol = auth_protocol
         self.require_secure_negotiate = require_secure_negotiate
-        self._domain_controller = None  # type: Optional[str]
+        self._domain_controller = domain_controller  # type: Optional[str]
         self._domain_cache = []  # type: List[DomainEntry]
         self._referral_cache = []  # type: List[ReferralEntry]
 
-        # This relies on other attributes so set this last
-        self.domain_controller = domain_controller
-
     @property
     def domain_controller(self):
         return self._domain_controller
@@ -319,7 +325,7 @@ def get_smb_tree(path, username=None, password=None, port=445, encrypt=None, con
 
 
 def register_session(server, username=None, password=None, port=445, encrypt=None, connection_timeout=60,
-                     connection_cache=None, auth_protocol='negotiate'):
+                     connection_cache=None, auth_protocol='negotiate', require_signing=True):
     """
     Creates an active connection and session to the server specified. This can be manually called to register the
     credentials of a specific server instead of defining it on the first function connecting to the server. The opened
@@ -330,13 +336,14 @@ def register_session(server, username=None, password=None, port=445, encrypt=Non
     :param username: Optional username to connect with. Required if no session has been registered for the server and
         Kerberos auth is not being used.
     :param password: Optional password to connect with.
-    :param port: The port to connect with.
+    :param port: The port to connect with. Defaults to 445.
     :param encrypt: Whether to force encryption or not, once this has been set to True the session cannot be changed
         back to False.
     :param connection_timeout: Override the timeout used for the initial connection.
     :param connection_cache: Connection cache to be used with
     :param auth_protocol: The protocol to use for authentication. Possible values are 'negotiate', 'ntlm' or
         'kerberos'. Defaults to 'negotiate'.
+    :param require_signing: Whether signing is required on SMB messages sent over this session. Defaults to True.
     :return: The Session that was registered or already existed in the pool.
     """
     connection_key = "%s:%s" % (server.lower(), port)
@@ -347,7 +354,7 @@ def register_session(server, username=None, password=None, port=445, encrypt=Non
 
     # Make sure we ignore any connections that may have had a closed connection
     if not connection or not connection.transport.connected:
-        connection = Connection(ClientConfig().client_guid, server, port)
+        connection = Connection(ClientConfig().client_guid, server, port, require_signing=require_signing)
         connection.connect(timeout=connection_timeout)
         connection_cache[connection_key] = connection
 
