@@ -2,8 +2,6 @@
 # Copyright: (c) 2019, Jordan Borean (@jborean93) <jborean93@gmail.com>
 # MIT License (see LICENSE or https://opensource.org/licenses/MIT)
 
-from __future__ import division
-
 import binascii
 import hashlib
 import hmac
@@ -49,7 +47,6 @@ from smbprotocol import (
 )
 
 from smbprotocol._text import (
-    to_native,
     to_text,
 )
 
@@ -86,7 +83,6 @@ from smbprotocol.structure import (
 )
 
 from smbprotocol.transport import (
-    _TimeoutError,
     Tcp,
 )
 
@@ -924,15 +920,16 @@ class Connection(object):
                         raise error
 
                     return new_response
-                elif status not in [NtStatus.STATUS_SUCCESS, NtStatus.STATUS_PENDING]:
-                    raise SMBResponseException(response)
                 else:
-                    break
+                    # now we have a retrieval request for the response, we can delete
+                    # the request from the outstanding requests
+                    message_id = request.message['message_id'].get_value()
+                    self.outstanding_requests.pop(message_id, None)
 
-        # now we have a retrieval request for the response, we can delete
-        # the request from the outstanding requests
-        message_id = request.message['message_id'].get_value()
-        del self.outstanding_requests[message_id]
+                    if status not in [NtStatus.STATUS_SUCCESS, NtStatus.STATUS_PENDING]:
+                        raise SMBResponseException(response)
+
+                    break
 
         return response
 
@@ -996,7 +993,7 @@ class Connection(object):
         actual = header['signature'].get_value()
         if actual != expected:
             raise SMBException("Server message signature could not be verified: %s != %s"
-                               % (to_native(binascii.hexlify(actual)), to_native(binascii.hexlify(expected))))
+                               % (binascii.hexlify(actual).decode(), binascii.hexlify(expected).decode()))
 
     def _check_worker_running(self):
         """ Checks that the message worker thread is still running and raises it's exception if it has failed. """
@@ -1117,7 +1114,18 @@ class Connection(object):
                 # https://github.com/jborean93/smbprotocol/issues/31
                 try:
                     b_msg = self.transport.recv(600)
-                except _TimeoutError:
+                except TimeoutError as ex:
+                    # Check if the connection has unanswered keepalive echo requests with the reserved field set.
+                    # When unanswered keep alive echo exists, the server did not respond withing two times the timeout.
+                    # We assume that the server connection is dead and close it.
+                    for r in self.outstanding_requests.values():
+                        if r.response is None and \
+                                r.message['command'].get_value() == Commands.SMB2_ECHO and \
+                                r.message['reserved'].get_value() == 1:
+                            # connection will be closed in finally block
+                            raise SMBConnectionClosed('Connection timed out. Server did not respond within timeout.') \
+                                from ex
+
                     log.debug("Sending SMB2 Echo to keep connection alive")
                     for sid in self.session_table.keys():
                         req = self.send(SMB2Echo(), sid=sid)
@@ -1189,7 +1197,7 @@ class Connection(object):
                         # When we send a ping in this thread we want to make sure it doesn't linger in the outstanding
                         # request queue.
                         if request.message['reserved'].get_value() == 1:
-                            del self.outstanding_requests[message_id]
+                            self.outstanding_requests.pop(message_id, None)
         except Exception as exc:
             # The exception is raised in _check_worker_running by the main thread when send/receive is called next.
             self._t_exc = exc
@@ -1376,33 +1384,30 @@ class Connection(object):
         :param message: The message being sent
         :return: The credit charge to set on the header
         """
-        credit_size = MAX_PAYLOAD_SIZE
-
         if (not self.supports_multi_credit) or (message.COMMAND == Commands.SMB2_CANCEL):
-            credit_charge = 0
+            return 0
+
         elif message.COMMAND == Commands.SMB2_READ:
-            max_size = message['length'].get_value() + \
-                       message['read_channel_info_length'].get_value() - 1
-            credit_charge = math.ceil(max_size / credit_size)
+            payload_size = message['length'].get_value() + message['read_channel_info_length'].get_value()
+
         elif message.COMMAND == Commands.SMB2_WRITE:
-            max_size = message['length'].get_value() + \
-                       message['write_channel_info_length'].get_value() - 1
-            credit_charge = math.ceil(max_size / credit_size)
+            payload_size = message['length'].get_value() + message['write_channel_info_length'].get_value()
+
         elif message.COMMAND == Commands.SMB2_IOCTL:
             max_in_size = len(message['buffer'])
             max_out_size = message['max_output_response'].get_value()
-            max_size = max(max_in_size, max_out_size) - 1
-            credit_charge = math.ceil(max_size / credit_size)
+            payload_size = max(max_in_size, max_out_size)
+
         elif message.COMMAND == Commands.SMB2_QUERY_DIRECTORY:
             max_in_size = len(message['buffer'])
             max_out_size = message['output_buffer_length'].get_value()
-            max_size = max(max_in_size, max_out_size) - 1
-            credit_charge = math.ceil(max_size / credit_size)
-        else:
-            credit_charge = 1
+            payload_size = max(max_in_size, max_out_size)
 
-        # python 2 returns a float where we need an integer
-        return int(credit_charge)
+        else:
+            payload_size = 1
+
+        credit_charge = (max(0, payload_size - 1) // MAX_PAYLOAD_SIZE) + 1
+        return credit_charge
 
 
 class Request(object):
