@@ -6,7 +6,6 @@ import binascii
 import errno
 import ntpath
 import os
-import six
 import socket
 
 from collections import (
@@ -19,7 +18,6 @@ from smbprotocol import (
 
 from smbprotocol._text import (
     to_bytes,
-    to_native,
     to_text,
 )
 
@@ -66,7 +64,7 @@ class SMBOSError(OSError, SMBException):
 
     def __init__(self, ntstatus, filename, filename2=None):
         self.ntstatus = ntstatus
-        self.filename2 = to_native(filename2) if filename2 else None
+        self.filename2 = str(filename2) if filename2 else None
 
         ntstatus_name = 'STATUS_UNKNOWN'
         for name, val in vars(NtStatus).items():
@@ -77,6 +75,7 @@ class SMBOSError(OSError, SMBException):
         error_details = {
             NtStatus.STATUS_OBJECT_NAME_NOT_FOUND: errno.ENOENT,
             NtStatus.STATUS_OBJECT_PATH_NOT_FOUND: errno.ENOENT,
+            NtStatus.STATUS_NOT_FOUND: errno.ENOENT,
             NtStatus.STATUS_OBJECT_NAME_COLLISION: errno.EEXIST,
             NtStatus.STATUS_PRIVILEGE_NOT_HELD: (errno.EACCES, "Required privilege not held"),
             NtStatus.STATUS_SHARING_VIOLATION: (errno.EPERM, "The process cannot access the file because it is being "
@@ -91,7 +90,7 @@ class SMBOSError(OSError, SMBException):
         if not isinstance(error_details, tuple):
             error_details = (error_details, os.strerror(error_details))
 
-        super(SMBOSError, self).__init__(error_details[0], error_details[1], to_native(filename))
+        super(SMBOSError, self).__init__(error_details[0], error_details[1], str(filename))
 
     def __str__(self):
         msg = "[Error {0}] [NtStatus 0x{1}] {2}: '{3}'".format(self.errno, format(self.ntstatus, 'x').zfill(8),
@@ -107,7 +106,7 @@ class SMBLinkRedirectionError(SMBException):
     @property
     def message(self):
         msg = "Encountered symlink at '%s' that points to '%s' which cannot be redirected: %s" \
-              % (to_native(self.path), to_native(self.target), to_native(self.args[0]))
+              % (str(self.path), str(self.target), str(self.args[0]))
         return msg
 
     @property
@@ -202,8 +201,7 @@ class _SMBErrorRegistry(type):
         return super(_SMBErrorRegistry, new_cls).__call__(header)
 
 
-@six.add_metaclass(_SMBErrorRegistry)
-class SMBResponseException(SMBException):
+class SMBResponseException(SMBException, metaclass=_SMBErrorRegistry):
     """Base SMB response status exception.
 
     This is the base exception that is used when processing the status of an SMB response received from the server.
@@ -273,7 +271,7 @@ class SMBResponseException(SMBException):
         if error_details:
             error_msg += " - %s" % ", ".join(error_details)
 
-        return to_native("Received unexpected status from the server: %s" % error_msg)
+        return "Received unexpected status from the server: %s" % error_msg
 
     @property
     def status(self):
@@ -283,6 +281,11 @@ class SMBResponseException(SMBException):
         return self.message
 
 
+class Unsuccessful(SMBResponseException):
+    _BASE_MESSAGE = "The requested operation was unsuccessful."
+    _STATUS_CODE = NtStatus.STATUS_UNSUCCESSFUL
+
+
 class NetworkNameDelegated(SMBResponseException):
     _BASE_MESSAGE = "The network name was deleted."
     _STATUS_CODE = NtStatus.STATUS_NETWORK_NAME_DELETED
@@ -336,6 +339,11 @@ class StoppedOnSymlink(SMBResponseException):
     _STATUS_CODE = NtStatus.STATUS_STOPPED_ON_SYMLINK
 
 
+class InvalidInfoClass(SMBResponseException):
+    _BASE_MESSAGE = "The specified information class is not a valid information class for the specified object."
+    _STATUS_CODE = NtStatus.STATUS_INVALID_INFO_CLASS
+
+
 class InfoLengthMismatch(SMBResponseException):
     _BASE_MESSAGE = "The specified information record length does not match the length that is required for the " \
                     "specified information class."
@@ -433,6 +441,11 @@ class EACorruptError(SMBResponseException):
     _STATUS_CODE = NtStatus.STATUS_EA_CORRUPT_ERROR
 
 
+class DeletePending(SMBResponseException):
+    _BASE_MESSAGE = "A non-close operation has been requested of a file object that has a delete pending."
+    _STATUS_CODE = NtStatus.STATUS_DELETE_PENDING
+
+
 class PrivilegeNotHeld(SMBResponseException):
     _BASE_MESSAGE = "A required privilege is not held by the client."
     _STATUS_CODE = NtStatus.STATUS_PRIVILEGE_NOT_HELD
@@ -454,6 +467,11 @@ class PasswordExpired(SMBResponseException):
     _STATUS_CODE = NtStatus.STATUS_PASSWORD_EXPIRED
 
 
+class NoneMapped(SMBResponseException):
+    _BASE_MESSAGE = "None of the information to be translated has been translated."
+    _STATUS_CODE = NtStatus.STATUS_NONE_MAPPED
+
+
 class InsufficientResources(SMBResponseException):
     _BASE_MESSAGE = "Insufficient system resources exist to complete the API."
     _STATUS_CODE = NtStatus.STATUS_INSUFFICIENT_RESOURCES
@@ -475,6 +493,11 @@ class PipeClosing(SMBResponseException):
     _STATUS_CODE = NtStatus.STATUS_PIPE_CLOSING
 
 
+class IOTimeout(SMBResponseException):
+    _BASE_MESSAGE = "The specified I/O operation was not completed before the time-out period expired."
+    _STATUS_CODE = NtStatus.STATUS_IO_TIMEOUT
+
+
 class PipeDisconnected(SMBResponseException):
     _BASE_MESSAGE = "The specified named pipe is in the disconnected state."
     _STATUS_CODE = NtStatus.STATUS_PIPE_DISCONNECTED
@@ -543,11 +566,22 @@ class PipeBroken(SMBResponseException):
     _STATUS_CODE = NtStatus.STATUS_PIPE_BROKEN
 
 
+class FSDriverRequired(SMBResponseException):
+    _BASE_MESSAGE = "A volume has been accessed for which a file system driver is required that " \
+                    "has not yet been loaded."
+    _STATUS_CODE = NtStatus.STATUS_FS_DRIVER_REQUIRED
+
+
 class UserSessionDeleted(SMBResponseException):
     _BASE_MESSAGE = "The remote user session has been deleted."
     _STATUS_CODE = NtStatus.STATUS_USER_SESSION_DELETED
 
 
+class InsuffServerResources(SMBResponseException):
+    _BASE_MESSAGE = "Insufficient server resources exist to complete the request."
+    _STATUS_CODE = NtStatus.STATUS_INSUFF_SERVER_RESOURCES
+
+
 class NotFound(SMBResponseException):
     _BASE_MESSAGE = "The object was not found."
     _STATUS_CODE = NtStatus.STATUS_NOT_FOUND
